@@ -319,6 +319,16 @@ where
     }
 }
 
+/// Split [`MultiHandler`] into parts.
+impl<K, H> IntoIterator for MultiHandler<K, H> {
+    type Item = <Self::IntoIter as Iterator>::Item;
+    type IntoIter = std::collections::hash_map::IntoIter<K, H>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.handlers.into_iter()
+    }
+}
+
 /// A [`IntoConnectionHandler`] for multiple other `IntoConnectionHandler`s.
 #[derive(Clone)]
 pub struct IntoMultiHandler<K, H> {
@@ -452,8 +462,7 @@ where
         self.upgrades
             .iter()
             .enumerate()
-            .map(|(i, (_, h))| iter::repeat(i).zip(h.protocol_info()))
-            .flatten()
+            .flat_map(|(i, (_, h))| iter::repeat(i).zip(h.protocol_info()))
             .map(|(i, h)| IndexedProtoName(i, h))
             .collect::<Vec<_>>()
             .into_iter()
